// -*- C++ -*-
/*!
 * @file Affinity.h
 * @brief  Processor affinity operation class
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2016
 *     Noriaki Ando
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

#ifndef COIL_AFFINITY_H
#define COIL_AFFINITY_H

#include <string>
#include <vector>
#include <coil/Singleton.h>
#include <coil/TimeValue.h>
#include <coil/Mutex.h>
#include <coil/Guard.h>

namespace coil
{
  typedef std::vector<unsigned int> CpuMask;
  /*!
   * @if jp
   * @brief ץCPU affinity
   * @prop cpu_mask ꤵƤ affinity  CPU ID
   *                std::vector<unsigned int> ֤롣
   * @return True: False: 
   * @else
   * @brief Getting current process CPU affinity
   * @prop cpu_mask Current CPU affinity mask is returned as CPU ID in
   *                std::vector<unsigned int>.
   * @return True: success, False: fail
   * @endif
   */
  bool getProcCpuAffinity(CpuMask& cpu_mask);

  /*!
   * @if jp
   * @brief ץCPU affinityꤹ
   * @prop cpu_mask ꤹ CPU affinity  CPU ID 
   *                std::vector<unsigned int> ꥹȤͿ
   * @return True: False: 
   * @else
   * @brief Setting process CPU affinity
   * @prop cpu_mask CPU affinity mask to be set is given with CPU ID in
   *                std::vector<unsigned int> list.
   * @return True: success, False: fail
   * @endif
   */
  bool setProcCpuAffinity(const CpuMask cpu_mask);

  /*!
   * @if jp
   * @brief ץCPU affinityʸꤹ
   * @prop cpu_mask ꤹ CPU affinity  CPU ID Υ޶ڤʸ
   *                ΥꥹȤͿ
   * @return True: False: 
   * @else
   * @brief Setting process CPU affinity with string
   * @prop cpu_mask CPU affinity mask to be set is given with comma
   *                separated CPU ID string.
   * @return True: success, False: fail
   * @endif
   */
  bool setProcCpuAffinity(std::string cpu_mask);

  /*!
   * @if jp
   * @brief åɤCPU affinity
   * @prop cpu_mask ꤵƤ affinity  CPU ID
   *                std::vector<unsigned int> ֤롣
   * @return True: False: 
   * @else
   * @brief Getting current process CPU affinity
   * @prop cpu_mask Current CPU affinity mask is returned as CPU ID in
   *                std::vector<unsigned int>.
   * @return True: success, False: fail
   * @endif
   */
  bool getThreadCpuAffinity(CpuMask& cpu_mask);

  /*!
   * @if jp
   * @brief åɤCPU affinityꤹ
   * @prop cpu_mask ꤹ CPU affinity  CPU ID 
   *                std::vector<unsigned int> ꥹȤͿ
   * @return True: False: 
   * @else
   * @brief Setting process CPU affinity
   * @prop cpu_mask CPU affinity mask to be set is given with CPU ID in
   *                std::vector<unsigned int> list.
   * @return True: success, False: fail
   * @endif
   */
  bool setThreadCpuAffinity(const CpuMask cpu_mask);

  /*!
   * @if jp
   * @brief åɤCPU affinityʸꤹ
   * @prop cpu_mask ꤹ CPU affinity  CPU ID Υ޶ڤʸ
   *                ΥꥹȤͿ
   * @return True: False: 
   * @else
   * @brief Setting process CPU affinity with string
   * @prop cpu_mask CPU affinity mask to be set is given with comma
   *                separated CPU ID string.
   * @return True: success, False: fail
   * @endif
   */
  bool setThreadCpuAffinity(std::string mask);

}; // namespace coil
#endif // COIL_AFFINITY_H
