// -*- C++ -*-
/*!
 * @file SharedMemory_win32.h
 * @brief SharedMemory class
 * @date $Date$
 * @author Nobuhiko Miyamoto <n-miyamoto@aist.go.jp>
 *
 * Copyright (C) 2017 Nobuhiko Miyamoto
 *     Robot Innovation Research Center,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

#ifndef COIL_SHAREDMEMORY_H
#define COIL_SHAREDMEMORY_H

#include <windows.h>

#include <string>
#include <coil/config_coil.h>

#define DEFAULT_MEMORY_SIZE 2097152



namespace coil
{
  /*!
   * @if jp
   *
   * @class SharedMemory
   * @brief SharedMemory NX
   *
   * @else
   *
   * @class SharedMemory
   * @brief SharedMemory class
   *
   * @endif
   */
  class SharedMemory
  {
  public:

    /*!
     * @if jp
     *
     * @brief RXgN^
     *
     * RXgN^B
     *
     *
     * @else
     *
     * @brief Constructor
     *
     * Constructor
     *
     *
     * @endif
     */
    SharedMemory();



    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * fXgN^B
     *
     * @else
     *
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~SharedMemory();

    /*!
     * @if jp
     *
     * @brief Rs[RXgN^
     *
     * Rs[RXgN^B
     *
     * @param rhs Rs[LIuWFNg
     *
     * @else
     *
     * @brief Copy Constructor
     *
     * Copy Constructor
     *
     * @param rhs shared memory object of copy source.
     *
     * @endif
     */
    SharedMemory(const SharedMemory& rhs);

    /*!
     * @if jp
     *
     * @brief Zq
     *
     * LIuWFNgRs[B
     *
     * @param rhs ILIuWFNg
     *
     * @return 
     *
     * @else
     *
     * @brief Assignment operator
     *
     * Copy a shared memory object.
     *
     * @param rhs shared memory object of assignment source.
     *
     * @return Assignment result.
     *
     * @endif
     */
    SharedMemory& operator=(const SharedMemory& rhs);


    /*!
     * @if jp
     *
     * @brief L̐
     *B
     *
     * @param shm_address L̎ʎq
     * @param memory_size L̃TCY
     *
     * @return 0: , -1: s
     *
     * @else
     *
     * @brief Create Shared Memory 
     *
     *
     * @param shm_address 
     * @param memory_size 
     *
     * @return 0: successful, -1: failed
     *
     * @endif
     */
    virtual int create(std::string shm_address,
                     int memory_size = DEFAULT_MEMORY_SIZE);


    /*!
     * @if jp
     *
     * @brief Lւ̃ANZX
     *B
     *
     * @param shm_address L̎ʎq
     *
     * @return 0: , -1: s
     *
     * @else
     *
     * @brief Open Shared Memory 
     *
     *
     * @param shm_address 
     * @param memory_size 
     *
     * @return 0: successful, -1: failed
     *
     * @endif
     */
	virtual int open(std::string shm_address,
		int memory_size = DEFAULT_MEMORY_SIZE);


    /*!
     * @if jp
     *
     * @brief Lւ̏
     *B
     *
     * @param data ރf[^
     *
     * @return 0: , -1: s
     *
     * @else
     *
     * @brief Write Shared Memory 
     *
     *
     * @param data 
     *
     * @return 0: successful, -1: failed
     *
     * @endif
     */
    virtual int write(const char *data, const int pos, const int size);


    /*!
     * @if jp
     *
     * @brief L̓ǂݍ
     *B
     *
     * @param data ǂݍރf[^
     *
     * @return 0: , -1: s
     *
     * @else
     *
     * @brief Read Shared Memory 
     *
     *
     * @param data 
     *
     * @return 0: successful, -1: failed
     *
     * @endif
     */
    virtual int read(char* data, const int pos, const int size);

    /*!
     * @if jp
     *
     * @brief L̍폜
     *B
     *
     *
     * @return 0: , -1: s
     *
     * @else
     *
     * @brief Close Shared Memory 
     *
     *
     *
     * @return 0: successful, -1: failed
     *
     * @endif
     */
    virtual int close();
    /*!
     * @if jp
     *
     * @brief ̃TCY擾
     *B
     *
     *
     * @return TCY
     *
     * @else
     *
     * @brief 
     *
     *
     *
     * @return size
     *
     * @endif
     */
    virtual unsigned int get_size();
    /*!
     * @if jp
     *
     * @brief ̃AhX擾
     *B
     *
     *
     * @return AhX
     *
     * @else
     *
     * @brief 
     *
     *
     *
     * @return address
     *
     * @endif
     */
	virtual std::string get_addresss();
    /*!
     * @if jp
     *
     * @brief ̎擾
     *
     *
     *
     * @return 
     *
     * @else
     *
     * @brief 
     *
     *
     *
     * @return memory
     *
     * @endif
     */
    virtual char *get_data();
    /*!
     * @if jp
     *
     * @brief t@C폜(Windows̏ꍇ͉Ȃ)
     *
     *
     *
     * @return 0: , -1: s
     *
     * @else
     *
     * @brief 
     *
     *
     *
     * @return 0: successful, -1: failed
     *
     * @endif
     */
    virtual int unlink();
     /*!
     * @if jp
     *
     * @brief Lɐς݂̊mF
     *
     *
     *
     * @return true: 쐬ς, false: 쐬
     *
     * @else
     *
     * @brief 
     *
     *
     *
     * @return
     *
     * @endif
     */
    virtual bool created();

  private:
    int m_memory_size;
    std::string m_shm_address;
    char *m_shm;
    HANDLE m_handle;
    bool m_file_create;
  };  // class SharedMemory

};  // namespace coil

#endif // SharedMemory_h
