// -*- C++ -*-
/*!
 * @file ExecutionContextBase.h
 * @brief ExecutionContext base class
 * @date $Date: 2008-01-14 07:48:55 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2007, 2011-2012
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: ExecutionContextBase.h 3097 2017-12-09 23:48:36Z n-ando $
 *
 */

#ifndef RTC_EXECUTIONCONTEXTBASE_H
#define RTC_EXECUTIONCONTEXTBASE_H

#include <coil/Factory.h>

#include <rtm/idl/RTCSkel.h>
#include <rtm/idl/OpenRTMSkel.h>
#include <rtm/Factory.h>
#include <rtm/ExecutionContextProfile.h>
#include <rtm/ExecutionContextWorker.h>

#ifdef WIN32
#pragma warning( disable : 4290 )
#endif

#define DEFAULT_EXECUTION_RATE 1000

namespace coil
{
  class Properties;
}

namespace RTC
{
  /*!
   * @if jp
   * @class ExecutionContextBase
   * @brief ExecutionContextѴ쥯饹
   *
   * ECμ饹Ǥϡδ쥯饹ѾECCORBAڥ졼
   * ʤФʤʤˡºݤ˥åư뤿
   * ᡢĤ«§ExecutionContextBaseδؿƤӽФɬפ
   * 롣ECCORBAڥ졼ϰʲΤΤꡢ줾
   * ExecutionContextBaseΥдؿбƤ롣
   *
   * - is_running(): ExecutionContextBase::isRunning()
   * - start(): ExecutionContextBase::start()
   * - stop(): ExecutionContextBase::stop()
   *
   * - get_rate(): ExecutionContextBase::gatRate()
   * - set_rate(): ExecutioinContextBase::setRate()
   *
   * - add_component(): ExecutionContextBase::addComponent()
   * - remove_component(): ExecutionContextBase::removeComponent()
   *
   * - activate_component(): ExecutionContextBase::activateComponent()
   * - deactivate_component(): ExecutionContextBase::deactivateComponent()
   * - reset_component(): ExecutionContextBase::resetComponent()
   *
   * - get_component_state(): ExecutionContextBase::getComponentState()
   * - get_kind(): ExecutionContextBase::getKind()
   * - get_profile(): ExecutionContextBase::getProfile()
   *
   * @par ¹Ծ֤˴طؿȼˡ
   * - is_running(): ExecutionContextBase::isRunning()
   * - start(): ExecutionContextBase::start()
   * - stop(): ExecutionContextBase::stop()
   *
   * ¹Ծ֤˴طؿϡis_running(), start(), stop() 3Ĥ
   * 롣ExecutionContextBaseǤñ running/stopped Υե饰
   * Ƥꡢstart/stopǥե饰ON/OFFڤؤis_running()Ǿɤ
   * ФԤäƤ롣̾ECμ饹Ǥϡprotected ʲۥ
   * дؿ onStarting(), onStarted(), onStopping(), onStopped() ؿ
   * ǡCORBAڥ졼ʲΤ褦˼ɬפ롣
   *
   * is_running() CORBAڥ졼Ǥϡñ
   * ExecutionContextBase  isRunning() ƤӽФǤ롣δؿ
   * ˴Ϣ protected ۴ؿonIsRunning() ѰդƤ뤬
   * ̾ä˼ɬפϤʤơߤ running/stopped ֤
   * 񤭴ˤδؿѤ뤳ȤǤ뤬侩Ϥʤ
   *
   * <pre>
   * public:
   *  CORBA::Boolean is_runing()
   *  {
   *    return ExecutionContextBase::isRunning();
   *  }
   * protected:
   *  CORBA::Boolean onIsRunning(CORBA::Boolean running)
   *  {
   *    return running;
   *  }
   * </pre>
   *
   * start(), stop() CORBAڥ졼Ǥϡ̾
   * ExecutionContextBase  start(), stop() ؿƤӽФ褦롣
   * δؿ˴Ϣ protected ۴ؿϡstart()  stop() 
   * ĤƤ줾2ĤŤĤ onStarting(), onStarted(), 
   * onStopping(), onStopped() ؿ롣ECμ饹ˤƤϡ
   * 줾ʲΤ褦˼롣
   *
   * <pre>
   *  RTC::ReturnCode_t start()
   *  {
   *    return ExecutionContextBase::start();
   *  }
   *  RTC::ReturnCode_t stop()
   *  {
   *    return ExecutionContextBase::stop();
   *  }
   * protected:
   *  RTC::ReturnCode_t onStarting()
   *  {
   *    RTC::ReturnCode_t ret = // åɤ򳫻Ϥʤ
   *    return ret;
   *  }
   *  RTC::ReturnCode_t onStarted()
   *  {
   *    RTC::ReturnCode_t ret = // åɤ򳫻Ϥʤ
   *    return ret;
   *  }
   *  RTC::ReturnCode_t onStopping()
   *  {
   *    // åɤߤʤ
   *    return retcode;
   *  }
   *  RTC::ReturnCode_t onStopped()
   *  {
   *    // åɤߤʤ
   *    return retcode;
   *  }
   * </pre>
   *
   * @par ¹Լ˴ؤؿȼˡ
   * - get_rate(): ExecutionContextBase::gatRate()
   * - set_rate(): ExecutioinContextBase::setRate()
   *
   * ¹Լ˴ؤؿ set_rate(), get_rate() 2ब롣
   * ¹ԥƥȤ⤷ set_rate() ˤꤵѤ
   * 硢ƥץ졼ȴؿ onSetRate() 򥪡С饤ɤ롣
   * onSetRate() ϰ double μꡢͤͤǤ
   * ȤݾڤƤ롣onSetRate() RTC::RTC_OK ʳ֤ͤ
   * 硢ECProfileμꤵͤݻ뤳Ȥݾڤ
   * 롣
   *
   * set_rate() Ʊ get_rate() ƤӽФonGetRate() ƤӽФ
   * ̾索С饤ɬפϤʤget_rate() 
   * ֤ͤѹ硢onGetRate() 򥪡С饤ɤ뤳ȤǤ
   * ͤ񤭴뤳ȤǤ롣Ͽ侩ʤ
   *
   * <pre>
   * public:
   *  RTC::ReturnCode_t set_rate(double rate)
   *  {
   *    return setRate(rate);
   *  }
   *  double get_rate(void) const
   *  {
   *    return getRate();
   *  }
   * protected:
   *  virtual RTC::ReturnCode_t onSetRate(double rate)
   *  {
   *    RTC::ReturnCode_t ret = // ꤹ벿餫ν
   *    if (ret != RTC::RTC_OK)
   *      {
   *        RTC_ERROR(("Error message"));
   *      }
   *    return ret;
   *  }
   *  virtual double onGetRate(rate)
   *  {
   *    // get_rate() ֤ͤù
   *    // ̾ϤδؿɬפϤʤ
   *    return rate;
   *  }
   * </pre>
   *
   * @par ݡͥȤɲäȺ˴ؤؿ
   * - add_component(): ExecutionContextBase::addComponent()
   * - remove_component(): ExecutionContextBase::removeComponent()
   *
   * ݡͥȤɲäȺ˴ؤؿϡadd_component(),
   * remove_component() ब롣¹ԥƥȤμ饹
   * ƤϡExecutionContextBase Τ줾 addComponent(),
   * removeComponent() ƤӽФǼԤδؿ˴Ϣ
   * protected ۴ؿ onAddingComponent(), onAddedComponent(),
   * onRemovingComponent(), onRemovedComponent() 4ढ롣
   * β۴ؿ̾索С饤ɤɬפϤʤѤϿ侩
   * ʤ
   *
   * <pre>
   * public:
   *  RTC::ReturnCode_t add_component(RTC::LightweightRTObject_ptr comp)
   *  {
   *    return ExecutionContextBase::addComponent(comp);
   *  }
   *  RTC::ReturnCode_t remove_component(RTC::LightweightRTObject_ptr comp)
   *  {
   *    return ExecutionContextBase::removeComponent(comp);
   *  }
   * protected:
   *  virtual RTC::ReturnCode_t
   *  onAddingComponent(RTC::LightweightRTObject rtobj)
   *  {
   *     // ݡͥɲû˼¹Ԥ򵭽
   *     // RTC::RTC_OK ʳ֤硢ݡͥȤɲäϹԤʤ
   *     return RTC::RTC_OK;
   *  }
   *  virtual RTC::ReturnCode_t
   *  onAddedComponent(RTC::LightweightRTObject rtobj)
   *  {
   *     // ݡͥɲû˼¹Ԥ򵭽
   *     // RTC::RTC_OK ʳ֤硢removeComponent() ƤӽФ졢
   *     // ɲä줿ݡͥȤ롣
   *     return RTC::RTC_OK;
   *  }
   *  virtual RTC::ReturnCode_t
   *  onRemovingComponent(RTC::LightweightRTObject rtobj)
   *  {
   *     // ݡͥȺ˼¹Ԥ򵭽
   *     // RTC::RTC_OK ʳ֤硢ݡͥȤκϹԤʤ
   *     return RTC::RTC_OK;
   *  }
   *  virtual RTC::ReturnCode_t
   *  onRemovedComponent(RTC::LightweightRTObject rtobj)
   *  {
   *     // ݡͥɲû˼¹Ԥ򵭽
   *     // RTC::RTC_OK ʳ֤硢addComponent() ƤӽФ졢
   *     // 줿ݡͥȤƤɲä롣
   *     return RTC::RTC_OK;
   *  }
   * </pre>
   *
   * @par ݡͥȤΥƥֲ˴ؤؿ
   * - activate_component(): ExecutionContextBase::activateComponent()
   * - deactivate_component(): ExecutionContextBase::deactivateComponent()
   * - reset_component(): ExecutionContextBase::resetComponent()
   *
   * ݡͥȤΥƥֲ˴ؤؿϡ
   * activate_component(), deactivate_component(), reset_component() 
   * ब롣¹ԥƥȤμ饹ˤƤϡ
   * ExecutionContextBase Τ줾 activateComponent(),
   * deactivateComponent(), resetComponent() ƤӽФǼԤ
   * δؿ˴Ϣ protected ۴ؿ
   * onActivatingComponent(), onAtivatingComponent(),
   * onActivatedComponent(), onDeactivatingComponent(),
   * onDeactivatedComponent(), onResettingComponent(),
   * onResetComponent() 6ढ롣β۴ؿ̾索
   * С饤ɤɬפϤʤѤϿ侩ʤ
   *
   * <pre>
   * public:
   *  RTC::ReturnCode_t add_component(RTC::LightweightRTObject_ptr comp)
   *  {
   *    return ExecutionContextBase::addComponent(comp);
   *  }
   *  RTC::ReturnCode_t remove_component(RTC::LightweightRTObject_ptr comp)
   *  {
   *    return ExecutionContextBase::removeComponent(comp);
   *  }
   * protected:
   *  virtual RTC::ReturnCode_t
   *  onAddingComponent(RTC::LightweightRTObject rtobj)
   *  {
   *    // ݡͥɲû˼¹Ԥ򵭽
   *    // RTC::RTC_OK ʳ֤硢ݡͥȤɲäϹԤʤ
   *    return RTC::RTC_OK;
   *  }
   *  virtual RTC::ReturnCode_t
   *  onAddedComponent(RTC::LightweightRTObject rtobj)
   *  {
   *    // ݡͥɲû˼¹Ԥ򵭽
   *    // RTC::RTC_OK ʳ֤硢removeComponent() ƤӽФ졢
   *    // ɲä줿ݡͥȤ롣
   *    return RTC::RTC_OK;
   *  }
   *  virtual RTC::ReturnCode_t
   *  onRemovingComponent(RTC::LightweightRTObject rtobj)
   *  {
   *    // ݡͥȺ˼¹Ԥ򵭽
   *    // RTC::RTC_OK ʳ֤硢ݡͥȤκϹԤʤ
   *    return RTC::RTC_OK;
   *  }
   *  virtual RTC::ReturnCode_t
   *  onRemovedComponent(RTC::LightweightRTObject rtobj)
   *  {
   *    // ݡͥɲû˼¹Ԥ򵭽
   *    // RTC::RTC_OK ʳ֤硢addComponent() ƤӽФ졢
   *    // 줿ݡͥȤƤɲä롣
   *    return RTC::RTC_OK;
   *  }
   * </pre>
   *
   * @par ¹ԥƥȤξ˴ؤؿ
   * - get_component_state(): ExecutionContextBase::getComponentState()
   * - get_kind(): ExecutionContextBase::getKind()
   * - get_profile(): ExecutionContextBase::getProfile()
   *
   * ¹ԥƥȤξ˴ؤؿϡget_component_state(),
   * get_kind(), get_profile() 3ब롣¹ԥƥȤμ
   * 饹ˤƤϡExecutionContextBase Τ줾
   * getComponentState(), getKind(), getProfile() ƤӽФǼ
   * Ԥδؿ˴Ϣ protected ۴ؿ
   * onGetComponentState(), onGetKind(), onGetProfile() 3ढ롣
   * β۴ؿ̾索С饤ɤɬפϤʤѤϿ侩
   * ֤ѹϡδؿŬڤ˼
   * 뤳ȤǡƤӽФ¦֤ͤ񤭤뤳ȤǤ롣
   *
   * <pre>
   * public:
   *  LifeCycleState get_component_state(RTC::LightweightRTObject_ptr comp)
   *  {
   *    return getComponentState(comp);
   *  }
   *  ExecutionKind PeriodicExecutionContext::get_kind()
   *  {
   *    return getKind();
   *  }
   *  ExecutionContextProfile* get_profile()
   *  {
   *    return getProfile();
   *  }
   *
   * protected:
   *  virtual LifeCycleState onGetComponentState(LifeCycleState state)
   *  { // ֤state񤭴Ϥδؿ
   *    return state;
   *  }
   *  virtual ExecutionKind onGetKind(ExecutionKind kind)
   *  { // ֤kind񤭴Ϥδؿ
   *    return kind;
   *  }
   *  virtual ExecutionContextProfile*
   *  onGetProfile(ExecutionContextProfile*& profile)
   *  { // ֤profile񤭴Ϥδؿ
   *    return profile;
   *  }
   * </pre>
   *
   * [ExecutionContextBase]
   * <>
   *  |
   *  +--(1 1) [ExecutionContextWorker ] : EC  worker
   *  |         <>
   *  |          +--(1 0..) [RTObjectStateMachine] : StateMachine
   *  |
   *  +--(1 1) [ExecutionContextProfile] : EC Profileδ
   * 
   * @since 0.4.0
   *
   * @else
   * @class ExecutionContextBase
   * @brief A base class for ExecutionContext
   *
   * A base class of ExecutionContext.
   *
   * @since 0.4.0
   *
   * @endif
   */
  class ExecutionContextBase
  {
  public:
    /*!
     * @if jp
     * @brief ۥǥȥ饯
     *
     * ۥǥȥ饯
     *
     * @else
     * @brief Virtual Destructor
     *
     * Virtual Destructor
     *
     * @endif
     */
    ExecutionContextBase(const char* name);

    /*!
     * @if jp
     * @brief ۥǥȥ饯
     *
     * ۥǥȥ饯
     *
     * @else
     * @brief Virtual Destructor
     *
     * Virtual Destructor
     *
     * @endif
     */
    virtual ~ExecutionContextBase(void);

    /*!
     * @if jp
     * @brief ExecutionContextνԤ
     *
     * ExecutionContextν
     *
     * @else
     * @brief Initialize the ExecutionContext
     *
     * This operation initialize the ExecutionContext
     *
     * @endif
     */
    virtual void init(coil::Properties& props);

    /*!
     * @if jp
     * @brief ݡͥȤХɤ롣
     *
     * ݡͥȤХɤ롣
     *
     * @else
     * @brief Bind the component.
     *
     * Bind the component.
     *
     * @endif
     */
    virtual RTC::ReturnCode_t bindComponent(RTC::RTObject_impl* rtc);

    //============================================================
    // implementation functions for EC's CORBA operation
    //============================================================
    /*!
     * @if jp
     * @brief ExecutionContext ¹Ծֳǧؿ
     *
     *  ExecutionContext  Runnning ֤ξ true ֤
     * ֤ͤϡstart() ؿƤФ줿ȤRunning֤Ȥʤ
     * true stop() ؿƤФ줿ȤStopped֤Ȥʤfalse֤
     *
     * @return ư (Running:trueStopped: false)
     *
     * @else
     *
     * @brief Check for ExecutionContext running state
     *
     * This operation shall return true if the context is in the
     * Running state.  After calling the start() of this class, this
     * function returns true, after calling to stop function of it,
     * this function returns false.
     *
     * @return Working status (Running:trueStopped:false)
     *
     * @endif
     */
    CORBA::Boolean isRunning();

    /*!
     * @if jp
     * @brief ExecutionContext μ¹Ԥ򳫻
     *
     * ExecutionContext μ¹Ծ֤ Runnning Ȥ뤿ΥꥯȤ
     * ȯԤ롣ExecutionContext ξ֤ܤ
     * ComponentAction::on_startup ƤӽФ롣äƤRTݡ
     * ͥȤޤ ExecutionContext 򳫻Ϥ뤳ȤϤǤ
     * ʤExecutionContext ʣ󳫻/ߤ򷫤֤ȤǤ롣
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Start the ExecutionContext
     *
     * Request that the context enter the Running state.  Once the
     * state transition occurs, the ComponentAction::on_startup
     * operation will be invoked.  An execution context may not be
     * started until the RT-Components that participate in it have
     * been initialized.  An execution context may be started and
     * stopped multiple times.
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t start(void);

    /*!
     * @if jp
     * @brief ExecutionContext μ¹Ԥ
     *
     * ExecutionContext ξ֤ Stopped Ȥ뤿ΥꥯȤȯԤ
     * 롣ܤȯϡComponentAction::on_shutdown Ƥӽ
     * 롣äƤRTݡͥȤλ
     * ExecutionContext ߤɬפ롣ExecutionContext ʣ
     * /ߤ򷫤֤ȤǤ롣
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Stop the ExecutionContext
     *
     * Request that the context enter the Stopped state.  Once the
     * transition occurs, the ComponentAction::on_shutdown operation
     * will be invoked.  An execution context must be stopped before
     * the RT components that participate in it are finalized.  An
     * execution context may be started and stopped multiple times.
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual RTC::ReturnCode_t stop(void);

    /*!
     * @if jp
     * @brief ExecutionContext μ¹Լ(Hz)
     *
     * Active ֤ˤRTݡͥȤ¹Ԥ(ñ:Hz)
     * 롣
     *
     * @return (ñ:Hz)
     *
     * @else
     *
     * @brief Get execution rate(Hz) of ExecutionContext
     *
     * This operation shall return the rate (in hertz) at which its
     * Active participating RTCs are being invoked.
     *
     * @return Execution cycle(Unit:Hz)
     *
     * @endif
     */
    double getRate(void) const;
    coil::TimeValue getPeriod(void) const;
 
    /*!
     * @if jp
     * @brief ExecutionContext μ¹Լ(Hz)ꤹ
     *
     * Active ֤ˤRTݡͥȤ¹Ԥ(ñ:Hz)ꤹ
     * 롣¹ԼѹϡDataFlowComponentAction 
     * on_rate_changed ˤäƳRTݡͥȤã롣
     *
     * @param rate (ñ:Hz)
     *
     * @return ReturnCode_t Υ꥿󥳡
     *         RTC_OK: ｪλ
     *         BAD_PARAMETER: ͤ
     *
     * @else
     *
     * @brief Set execution rate(Hz) of ExecutionContext
     *
     * This operation shall set the rate (in hertz) at which this
     * contexts Active participating RTCs are being called.  If the
     * execution kind of the context is PERIODIC, a rate change shall
     * result in the invocation of on_rate_changed on any RTCs
     * realizing DataFlowComponentAction that are registered with any
     * RTCs participating in the context.
     *
     * @param rate Execution cycle(Unit:Hz)
     *
     * @return The return code of ReturnCode_t type
     *         RTC_OK: Succeed
     *         BAD_PARAMETER: Invalid value. The value might be negative.
     *
     * @endif
     */
    RTC::ReturnCode_t setRate(double rate);

    /*!
     * @if jp
     * @brief RTݡͥȤɲä
     *
     * ꤷRTݡͥȤ򻲲üԥꥹȤɲä롣ɲä줿RT
     * ݡͥȤ attach_context ƤФ졢Inactive ֤ܤ롣
     * ꤵ줿RTݡͥȤnullξϡBAD_PARAMETER ֤
     * 롣ꤵ줿RTݡͥȤ DataFlowComponent ʳξϡ
     * BAD_PARAMETER ֤롣
     *
     * @param comp ɲоRTݡͥ
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Add an RT-component
     *
     * The operation causes the given RTC to begin participating in
     * the execution context.  The newly added RTC will receive a call
     * to LightweightRTComponent::attach_context and then enter the
     * Inactive state.  BAD_PARAMETER will be invoked, if the given
     * RT-Component is null or if the given RT-Component is other than
     * DataFlowComponent.
     *
     * @param comp The target RT-Component for add
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t addComponent(RTC::LightweightRTObject_ptr comp);

    /*!
     * @if jp
     * @brief RTݡͥȤ򻲲üԥꥹȤ
     *
     * ꤷRTݡͥȤ򻲲üԥꥹȤ롣줿
     * RTݡͥȤ detach_context ƤФ롣ꤵ줿RTݡ
     * ͥȤüԥꥹȤϿƤʤϡBAD_PARAMETER 
     * 롣
     *
     * @param comp оRTݡͥ
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Remove the RT-Component from participant list
     *
     * This operation causes a participant RTC to stop participating in the
     * execution context.
     * The removed RTC will receive a call to
     * LightweightRTComponent::detach_context.
     * BAD_PARAMETER will be returned, if the given RT-Component is not
     * participating in the participant list.
     *
     * @param comp The target RT-Component for delete
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t removeComponent(RTC::LightweightRTObject_ptr comp);

    /*!
     * @if jp
     * @brief RTݡͥȤ򥢥ƥֲ
     *
     * Inactive ֤ˤRTݡͥȤActive ܤƥ
     * ֲ롣ƤФ줿̡on_activate ƤӽФ롣
     * ꤷRTݡͥȤüԥꥹȤ˴ޤޤʤϡ
     * BAD_PARAMETER ֤롣ꤷRTݡͥȤξ֤
     * Inactive ʳξϡPRECONDITION_NOT_MET ֤롣
     *
     * @param comp ƥֲоRTݡͥ
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Activate an RT-component
     *
     * The given participant RTC is Inactive and is therefore not
     * being invoked according to the execution contexts execution
     * kind. This operation shall cause the RTC to transition to the
     * Active state such that it may subsequently be invoked in this
     * execution context.  The callback on_activate shall be called as
     * a result of calling this operation. This operation shall not
     * return until the callback has returned, and shall result in an
     * error if the callback does.
     *
     * @param comp The target RT-Component for activation
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t activateComponent(RTC::LightweightRTObject_ptr comp);

    /*!
     * @if jp
     * @brief RTݡͥȤ󥢥ƥֲ
     *
     * Inactive ֤ˤRTݡͥȤ󥢥ƥֲInactive
     * ܤ롣ƤФ줿̡on_deactivate ƤӽФ
     * 롣ꤷRTݡͥȤüԥꥹȤ˴ޤޤʤϡ
     * BAD_PARAMETER ֤롣ꤷRTݡͥȤξ֤
     * Active ʳξϡPRECONDITION_NOT_MET ֤롣
     *
     *
     *


     * |


     
     * @param comp 󥢥ƥֲоRTݡͥ
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Deactivate an RT-component
     *
     * The given RTC is Active in the execution context. Cause it to
     * transition to the Inactive state such that it will not be
     * subsequently invoked from the context unless and until it is
     * activated again.  The callback on_deactivate shall be called as
     * a result of calling this operation. This operation shall not
     * return until the callback has returned, and shall result in an
     * error if the callback does.
     *
     * @param comp The target RT-Component for deactivate
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t deactivateComponent(RTC::LightweightRTObject_ptr comp);

    /*!
     * @if jp
     * @brief RTݡͥȤꥻåȤ
     *
     * Error ֤RTݡͥȤߤ롣ƤФ줿
     * ̡on_reset ƤӽФ롣ꤷRTݡͥȤüԥ
     * Ȥ˴ޤޤʤϡBAD_PARAMETER ֤롣ꤷRT
     * ݡͥȤξ֤ Error ʳξϡPRECONDITION_NOT_MET 
     * 롣
     *
     * @param comp ꥻåоRTݡͥ
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Reset the RT-component
     *
     * Attempt to recover the RTC when it is in Error.  The
     * ComponentAction::on_reset callback shall be invoked. This
     * operation shall not return until the callback has returned, and
     * shall result in an error if the callback does. If possible, the
     * RTC developer should implement that callback such that the RTC
     * may be returned to a valid state.
     *
     * @param comp The target RT-Component for reset
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t resetComponent(RTC::LightweightRTObject_ptr comp);

    /*!
     * @if jp
     * @brief RTݡͥȤξ֤
     *
     * ꤷRTݡͥȤξ(LifeCycleState)롣ꤷ
     * RTݡͥȤüԥꥹȤ˴ޤޤʤϡ
     * UNKNOWN_STATE ֤롣
     *
     * @param comp ּоRTݡͥ
     *
     * @return ߤξ(LifeCycleState)
     *
     * @else
     *
     * @brief Get RT-component's state
     *
     * This operation shall report the LifeCycleState of the given
     * participant RTC.  UNKNOWN_STATE will be returned, if the given
     * RT-Component is not inclued in the participant list.
     *
     * @param comp The target RT-Component to get the state
     *
     * @return The current state of the target RT-Component(LifeCycleState)
     *
     * @endif
     */
    RTC::LifeCycleState getComponentState(RTC::LightweightRTObject_ptr comp);
    const char* getStateString(RTC::LifeCycleState state);
    
    /*!
     * @if jp
     * @brief ExecutionKind 
     *
     *  ExecutionContext  ExecutionKind 
     *
     * @return ExecutionKind
     *
     * @else
     *
     * @brief Get the ExecutionKind
     *
     * This operation shall report the execution kind of the execution
     * context.
     *
     * @return ExecutionKind
     *
     * @endif
     */
    RTC::ExecutionKind getKind(void) const;

    /*!
     * @if jp
     * @brief Profile
     *
     * RTC::ExecutionContextProfile 롣
     * ExecutionContextProfile νͭϸƤӽФ¦ˤ롣줿
     * ֥Ȥפˤʤä硢ƤӽФ¦Ǥ餦
     *
     * @return RTC::ExecutionContextProfile
     *
     * @else
     * @brief Getting Profile
     *
     * This function gets RTC::ExecutionContextProfile.  The ownership
     * of the obtained ExecutionContextProfile is given to caller. The
     * caller should release obtained object when it is unneccessary
     * anymore.
     *
     * @return RTC::ExecutionContextProfile
     *
     * @endif
     */
    RTC::ExecutionContextProfile* getProfile(void);

    //============================================================
    // Delegated functions to ExecutionContextProfile
    //============================================================
    /*!
     * @if jp
     * @brief CORBA ֥ȻȤμ
     *
     * ܥ֥Ȥ ExecutioncontextService ȤƤ CORBA ֥
     * ȻȤ롣
     *
     * @return CORBA ֥Ȼ
     *
     * @else
     * @brief Get the reference to the CORBA object
     *
     * Get the reference to the CORBA object as
     * ExecutioncontextService of this object.
     *
     * @return The reference to CORBA object
     *
     * @endif
     */
    void setObjRef(RTC::ExecutionContextService_ptr ec_ptr);

    /*!
     * @if jp
     * @brief CORBA ֥ȻȤμ
     *
     * ܥ֥Ȥ ExecutioncontextService ȤƤ CORBA ֥
     * ȻȤ롣
     *
     * @return CORBA ֥Ȼ
     *
     * @else
     *
     * Get the reference to the CORBA object as
     * ExecutioncontextService of this object.
     *
     * @return The reference to CORBA object
     *
     * @endif
     */
    RTC::ExecutionContextService_ptr getObjRef(void) const;

    /*!
     * @if jp
     * @brief ExecutionKind ʸ󲽤
     *
     * RTC::ExecutionKind Ƥ PERIODIC, EVENT_DRIVEN,
     * OTHER ʸ󲽤롣
     *
     * @param kind ExecutionKind
     * @return ʸ󲽤줿ExecutionKind
     *
     * @else
     *
     * @brief Converting ExecutionKind enum to string 
     *
     * This function converts enumeration (PERIODIC, EVENT_DRIVEN,
     * OTHER) defined in RTC::ExecutionKind to string.
     *
     * @param kind ExecutionKind
     * @return String of ExecutionKind
     *
     * @endif
     */
    const char* getKindString(RTC::ExecutionKind kind) const;

    /*!
     * @if jp
     * @brief ExecutionKind ꤹ
     *
     *  ExecutionContext  ExecutionKind ꤹ
     *
     * @param kind ExecutionKind
     *
     * @else
     *
     * @brief Set the ExecutionKind
     *
     * This operation sets the kind of the execution context.
     *
     * @param kind ExecutionKind
     *
     * @endif
     */
    RTC::ReturnCode_t setKind(RTC::ExecutionKind kind);

    /*!
     * @if jp
     * @brief OwnerݡͥȤ򥻥åȤ롣
     *
     * ECOwnerȤʤRTC򥻥åȤ롣
     *
     * @param comp OwnerȤʤRTݡͥ
     * @return ReturnCode_t Υ꥿󥳡
     * @else
     * @brief Setting owner component of the execution context
     *
     * This function sets an RT-Component to be owner of the execution context.
     *
     * @param comp an owner RT-Component of this execution context
     * @return The return code of ReturnCode_t type
     * @endif
     */
    RTC::ReturnCode_t setOwner(RTC::LightweightRTObject_ptr comp);

    /*!
     * @if jp
     * @brief OwnerݡͥȤλȤ
     *
     * ECOwnerǤRTCλȤ롣
     *
     * @return OwnerRTݡͥȤλ
     * @else
     * @brief Getting a reference of the owner component
     *
     * This function returns a reference of the owner RT-Component of
     * this execution context
     *
     * @return a reference of the owner RT-Component
     * @endif
     */
    const RTC::RTObject_ptr getOwner() const;

    /*!
     * @if jp
     * @brief RTݡͥȤλüԥꥹȤ
     *
     * ϿƤ뻲üRTCΥꥹȤ롣
     *
     * @return üRTCΥꥹ
     *
     * @else
     *
     * @brief Getting participant RTC list
     *
     * This function returns a list of participant RTC of the execution context.
     *
     * @return Participants RTC list
     *
     * @endif
     */
    const RTC::RTCList& getComponentList() const;

    /*!
     * @if jp
     * @brief Properties򥻥åȤ
     *
     * ExecutionContextProfile::properties 򥻥åȤ롣
     *
     * @param props ExecutionContextProfile::properties ˥åȤ
     *              ѥƥ
     *
     * @else
     * @brief Setting Properties
     *
     * This function sets ExecutionContextProfile::properties by
     * coil::Properties.
     *
     * @param props Properties to be set to
     *              ExecutionContextProfile::properties.
     *
     * @endif
     */
    void setProperties(coil::Properties& props);

    /*!
     * @if jp
     * @brief Properties
     *
     * ExecutionContextProfile::properties 롣
     *
     * @return coil::PropertiesѴ줿
     *              ExecutionContextProfile::properties
     *
     * @else
     * @brief Setting Properties
     *
     * This function sets ExecutionContextProfile::properties by
     * coil::Properties.
     *
     * @param props Properties to be set to ExecutionContextProfile::properties.
     *
     * @endif
     */
    const coil::Properties getProperties() const;

    /*!
     * @if jp
     * @brief Profile
     *
     * RTC::ExecutionContextProfile 롣
     *
     * @return RTC::ExecutionContextProfile
     *
     * @else
     * @brief Getting Profile
     *
     * This function gets RTC::ExecutionContextProfile.
     *
     * @return RTC::ExecutionContextProfile
     *
     * @endif
     */
    const RTC::ExecutionContextProfile& getProfile(void) const;
    // end of delegated functions to ExecutionContextProfile
    //============================================================

    //============================================================
    // Delegated functions to ExecutionContextWorker
    //============================================================
    bool isAllCurrentState(RTC::LifeCycleState state)
    {
      return m_worker.isAllCurrentState(state);
    };
    bool isAllNextState(RTC::LifeCycleState state)
    {
      return m_worker.isAllNextState(state);
    };
    bool isOneOfCurrentState(RTC::LifeCycleState state)
    {
      return m_worker.isOneOfCurrentState(state);
    };
    bool isOneOfNextState(RTC::LifeCycleState state)
    {
      return m_worker.isOneOfNextState(state);
    };
    
    void invokeWorker()       { m_worker.invokeWorker(); }
    void invokeWorkerPreDo()  { m_worker.invokeWorkerPreDo(); }
    void invokeWorkerDo()     { m_worker.invokeWorkerDo(); }
    void invokeWorkerPostDo() { m_worker.invokeWorkerPostDo(); }


  protected:
    // template virtual functions related to start/stop
    virtual bool onIsRunning(bool running) { return running; }
    virtual RTC::ReturnCode_t onStarting() { return RTC::RTC_OK; }
    virtual RTC::ReturnCode_t onStarted() { return RTC::RTC_OK; }
    virtual RTC::ReturnCode_t onStopping() { return RTC::RTC_OK; }
    virtual RTC::ReturnCode_t onStopped() { return RTC::RTC_OK; }

    // template virtual functions getting/setting execution rate
    virtual double onGetRate(double rate) const { return rate; }
    virtual double onSettingRate(double rate) { return rate; }
    virtual RTC::ReturnCode_t onSetRate(double rate) { return RTC::RTC_OK; }

    // template virtual functions adding/removing component
    virtual RTC::ReturnCode_t
    onAddingComponent(RTC::LightweightRTObject_ptr rtobj)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onAddedComponent(RTC::LightweightRTObject_ptr rtobj)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onRemovingComponent(RTC::LightweightRTObject_ptr rtobj)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onRemovedComponent(RTC::LightweightRTObject_ptr rtobj)
    {
      return RTC::RTC_OK;
    }

    // template virtual functions related to activation/deactivation/reset
    virtual RTC::ReturnCode_t
    onActivating(RTC::LightweightRTObject_ptr comp)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onWaitingActivated(RTC_impl::RTObjectStateMachine* comp, long int count)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onActivated(RTC_impl::RTObjectStateMachine* comp,
                                     long int count)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onDeactivating(RTC::LightweightRTObject_ptr comp)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onWaitingDeactivated(RTC_impl::RTObjectStateMachine* comp, long int count)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onDeactivated(RTC_impl::RTObjectStateMachine* comp, long int count)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t onResetting(RTC::LightweightRTObject_ptr comp)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onWaitingReset(RTC_impl::RTObjectStateMachine* comp, long int count)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onReset(RTC_impl::RTObjectStateMachine* comp, long int count)
    {
      return RTC::RTC_OK;
    }

    virtual RTC::LifeCycleState
    onGetComponentState(RTC::LifeCycleState state)
    {
      return state;
    }
    virtual RTC::ExecutionKind
    onGetKind(RTC::ExecutionKind kind) const
    {
      return kind;
    }
    virtual RTC::ExecutionContextProfile*
    onGetProfile(RTC::ExecutionContextProfile*& profile)
    {
      return profile;
    }

  private:
    /*!
     * @if jp
     * @brief Properties¹ԥƥȤ򥻥åȤ
     * @else
     * @brief Setting execution rate from given properties.
     * @endif
     */
    bool setExecutionRate(coil::Properties& props);
    /*!
     * @if jp
     * @brief Propertiesܥ⡼ɤ򥻥åȤ
     * @else
     * @brief Setting state transition mode from given properties.
     * @endif
     */
    bool setTransitionMode(coil::Properties& props,
                           const char* key, bool& flag);
    /*!
     * @if jp
     * @brief PropertiesTimeout򥻥åȤ
     * @else
     * @brief Setting state transition timeout from given properties.
     * @endif
     */
    bool setTimeout(coil::Properties& props, const char* key,
                    coil::TimeValue& timevalue);

    RTC::ReturnCode_t waitForActivated(RTC_impl::RTObjectStateMachine* rtobj);
    RTC::ReturnCode_t waitForDeactivated(RTC_impl::RTObjectStateMachine* rtobj);
    RTC::ReturnCode_t waitForReset(RTC_impl::RTObjectStateMachine* rtobj);
    
  protected:
    mutable RTC::Logger rtclog;

    RTC_impl::ExecutionContextWorker m_worker;
    RTC_impl::ExecutionContextProfile m_profile;

    coil::TimeValue m_activationTimeout;
    coil::TimeValue m_deactivationTimeout;
    coil::TimeValue m_resetTimeout;

    bool m_syncActivation;
    bool m_syncDeactivation;
    bool m_syncReset;
  };  // class ExecutionContextBase

  typedef coil::GlobalFactory<ExecutionContextBase> ExecutionContextFactory;

#if defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__)
  EXTERN template class DLL_PLUGIN coil::GlobalFactory<ExecutionContextBase>;
#endif
};  // namespace RTC

#ifdef WIN32
#pragma warning( default : 4290 )
#endif

#endif // RTC_EXECUTIONCONTEXTBASE_H
