// -*- C++ -*-
/*!
 * @file ExecutionContextWorker.h
 * @brief ExecutionContext's state machine worker class
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2011
 *     Noriaki Ando
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

#ifndef RTC_EXECUTIONCONTEXTWORKER_H
#define RTC_EXECUTIONCONTEXTWORKER_H

#include <coil/Mutex.h>
#include <coil/Condition.h>
#include <vector>

#include <rtm/idl/RTCSkel.h>
#include <rtm/SystemLogger.h>
//#include <rtm/StateMachine.h>
//#include <rtm/RTObjectStateMachine.h>

#define NUM_OF_LIFECYCLESTATE 4

#ifdef WIN32
#pragma warning( disable : 4290 )
#endif

namespace RTC
{
  class RTObject_impl;
};
namespace RTC_impl
{
  class RTObjectStateMachine;
  typedef RTC::LightweightRTObject_ptr LightweightRTObject_ptr;
  typedef RTC::LightweightRTObject_var LightweightRTObject_var;
  /*!
   * @if jp
   * @class ExecutionContextWorker
   * @brief ExecutionContextWorker 饹
   *
   * RTCξ֥ޥ󡦥ץ饹 RTObjectStateMachine ݻ
   * Хåºݤ˥뤹뤿Υ饹󥿡ե
   * 饹ECμԤϡExecutionContextBase Ѿѿ
   * ȤݻƤ뤳 ExecutionContextWorker 饹Υ󥿡
   * եͳRTObjectΥХå򥳡뤹뤳Ȥˤʤ롣
   *
   * - ECΥ֥ȥե󥹤Υåå
   *   - void setECRef(RTC::ExecutionContextService_ptr ref);
   *   - RTC::ExecutionContextService_ptr getECRef();
   *   ECζݥ饹ˤϡƥֲECΥ֥ȥե
   *   󥹤setECRef() ǥåȤ롣
   *
   * - ECμ¹ԡߴطؿ
   *   - CORBA::Boolean isRunning(void);
   *   - RTC::ReturnCode_t start(void);
   *   - RTC::ReturnCode_t stop(void);
   *
   * - RTCΥƥֲ󥢥ƥֲؿ
   *   - RTC::ReturnCode_t activateComponent(RTC::LightweightRTObject_ptr comp,
   *                                         RTObjectStateMachine*& rtobj);
   *   - RTC::ReturnCode_t waitActivateComplete(RTObjectStateMachine*& rtobj,
   *                                            coil::TimeValue timeout = 1.0,
   *                                            long int cycle = 1000);
   *
   *   - RTC::ReturnCode_t deactivateComponent(RTC::LightweightRTObject_ptr comp,  *                                           RTObjectStateMachine*& rtobj);
   *   - RTC::ReturnCode_t waitDeactivateComplete(RTObjectStateMachine*& rtobj,
   *                                              coil::TimeValue timeout = 1.0,
   *                                              long int cycle = 1000);
   *   -  RTC::ReturnCode_t resetComponent(RTC::LightweightRTObject_ptr com,
   *                                       RTObjectStateMachine*& rtobj);
   *   -  RTC::ReturnCode_t waitResetComplete(RTObjectStateMachine*& rtobj,
   *                                          coil::TimeValue timeout = 1.0,
   *                                          long int cycle = 1000);
   *   - RTC::LifeCycleState getComponentState(RTC::LightweightRTObject_ptr comp);
   *   - const char* getStateString(RTC::LifeCycleState state)
   *
   * - RTCɲáϴؿ
   *   - RTC::ReturnCode_t addComponent(RTC::LightweightRTObject_ptr comp);
   *   - RTC::ReturnCode_t bindComponent(RTC::RTObject_impl* rtc);
   *   - RTC::ReturnCode_t removeComponent(RTC::LightweightRTObject_ptr comp);
   *   - RTObjectStateMachine* findComponent(RTC::LightweightRTObject_ptr comp);
   *
   * - 䤤碌ϴؿ
   *   - bool isAllCurrentState(RTC::LifeCycleState state);
   *   - bool isAllNextState(RTC::LifeCycleState state);
   *   - bool isOneOfCurrentState(RTC::LifeCycleState state);
   *   - bool isOneOfNextState(RTC::LifeCycleState state);
   *
   * - WorkerζưӥХå
   *   - void invokeWorker();
   *   - void invokeWorkerPreDo();
   *   - void invokeWorkerDo();
   *   - void invokeWorkerPostDo();
   *
   * @since 1.2
   *
   * @else
   * @class ExecutionContextWorker
   * @brief ExecutionContextWorker class
   *
   * Periodic Sampled Data Processing (for the execution cycles)
   * ExecutionContext class
   *
   * @since 0.4.0
   *
   * @endif
   */
  class ExecutionContextWorker
  {
    typedef coil::Guard<coil::Mutex> Guard;

  public:
    /*!
     * @if jp
     * @brief ǥեȥ󥹥ȥ饯
     *
     * ǥեȥ󥹥ȥ饯
     * ץե˰ʲιܤꤹ롣
     *  - kind : PERIODIC
     *  - rate : 0.0
     *
     * @else
     * @brief Default Constructor
     *
     * Default Constructor
     * Set the following items to profile.
     *  - kind : PERIODIC
     *  - rate : 0.0
     *
     * @endif
     */
    ExecutionContextWorker();


    /*!
     * @if jp
     * @brief ǥȥ饯
     *
     * ǥȥ饯
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~ExecutionContextWorker(void);

    //============================================================
    // Object reference to EC
    //============================================================
    void setECRef(RTC::ExecutionContextService_ptr ref);
    RTC::ExecutionContextService_ptr getECRef();

    //============================================================
    // ExecutionContext
    //============================================================
    /*!
     * @if jp
     * @brief ExecutionContext ¹Ծֳǧؿ
     *
     *  ExecutionContext  Runnning ֤ξ true ֤
     * Executioncontext  Running δ֡ Executioncontext ˻ä
     * ƤƤΥƥRTݡͥȤExecutionContext μ
     * Լ˱Ƽ¹Ԥ롣
     *
     * @return ֳǧؿ(ư:true:false)
     *
     * @else
     *
     * @brief Check for ExecutionContext running state
     *
     * This operation shall return true if the context is in the
     * Running state.  While the context is Running, all Active RTCs
     * participating in the context shall be executed according to the
     * contexts execution kind.
     *
     * @return Check state function (Running:trueStopping:false)
     *
     * @endif
     */
    CORBA::Boolean isRunning(void);

    /*!
     * @if jp
     * @brief ExecutionContext μ¹Ԥ򳫻
     *
     * ExecutionContext μ¹Ծ֤ Runnning Ȥ뤿ΥꥯȤ
     * ȯԤ롣ExecutionContext ξ֤ܤ
     * ComponentAction::on_startup ƤӽФ롣äƤRTݡ
     * ͥȤޤ ExecutionContext 򳫻Ϥ뤳ȤϤǤ
     * ʤExecutionContext ʣ󳫻/ߤ򷫤֤ȤǤ롣
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Start the ExecutionContext
     *
     * Request that the context enter the Running state.  Once the
     * state transition occurs, the ComponentAction::on_startup
     * operation will be invoked.  An execution context may not be
     * started until the RT-Components that participate in it have
     * been initialized.  An execution context may be started and
     * stopped multiple times.
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t start(void);

    /*!
     * @if jp
     * @brief ExecutionContext μ¹Ԥ
     *
     * ExecutionContext ξ֤ Stopped Ȥ뤿ΥꥯȤȯԤ
     * 롣ܤȯϡComponentAction::on_shutdown Ƥӽ
     * 롣äƤRTݡͥȤλ
     * ExecutionContext ߤɬפ롣ExecutionContext ʣ
     * /ߤ򷫤֤ȤǤ롣
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Stop the ExecutionContext
     *
     * Request that the context enter the Stopped state.  Once the
     * transition occurs, the ComponentAction::on_shutdown operation
     * will be invoked.  An execution context must be stopped before
     * the RT components that participate in it are finalized.  An
     * execution context may be started and stopped multiple times.
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t stop(void);

    RTC::ReturnCode_t startThread(void);

    RTC::ReturnCode_t stopThread(void);

    
    /*!
     * @if jp
     * @brief ExecutionContext μ¹Լѹ줿
     *
     * ExecutionContext μ¹Լѹ줿ΥХåƤ
     * ФECμ饹 set_rate() ڥ졼󤬸ƤӽФ줿
     * 硢ExecutionContextProfile::setRate() ƤӽФ졢ץե
     * μͤȤȤˡδؿƤФ졢
     * RTObjectStateMachine::onRateChagned() ƤӽФ롣
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Execution rate of ExecutionContext chnaged
     *
     * A call back for execution rate chang when ExecutionContext's
     * execution rate. If set_rate() operation is called,
     * ExecutionContextProfile::setRate() will be called, execution
     * rate in the profile will be updated, and
     * RTObjectStateMachine::onRateChagned() will be called with this
     * member function.
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t rateChanged();

    /*!
     * @if jp
     * @brief RTݡͥȤ򥢥ƥֲ
     *
     * Inactive ֤ˤRTݡͥȤActive ܤƥ
     * ֲ롣ƤФ줿̡on_activate ƤӽФ롣
     * ꤷRTݡͥȤüԥꥹȤ˴ޤޤʤϡ
     * BAD_PARAMETER ֤롣ꤷRTݡͥȤξ֤
     * Inactive ʳξϡPRECONDITION_NOT_MET ֤롣
     *
     * @param comp ƥֲоRTݡͥ
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Activate an RT-component
     *
     * The given participant RTC is Inactive and is therefore not
     * being invoked according to the execution contexts execution
     * kind. This operation shall cause the RTC to transition to the
     * Active state such that it may subsequently be invoked in this
     * execution context.  The callback on_activate shall be called as
     * a result of calling this operation. This operation shall not
     * return until the callback has returned, and shall result in an
     * error if the callback does.
     *
     * @param comp The target RT-Component for activation
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t activateComponent(RTC::LightweightRTObject_ptr comp,
                                        RTObjectStateMachine*& rtobj);
    RTC::ReturnCode_t waitActivateComplete(RTObjectStateMachine*& rtobj,
                                           coil::TimeValue timeout = 1.0,
                                           long int cycle = 1000);
    /*!
     * @if jp
     * @brief RTݡͥȤ󥢥ƥֲ
     *
     * Inactive ֤ˤRTݡͥȤ󥢥ƥֲInactive
     * ܤ롣ƤФ줿̡on_deactivate ƤӽФ
     * 롣ꤷRTݡͥȤüԥꥹȤ˴ޤޤʤϡ
     * BAD_PARAMETER ֤롣ꤷRTݡͥȤξ֤
     * Active ʳξϡPRECONDITION_NOT_MET ֤롣
     *a
     * @param comp 󥢥ƥֲоRTݡͥ
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Deactivate an RT-component
     *
     * The given RTC is Active in the execution context. Cause it to
     * transition to the Inactive state such that it will not be
     * subsequently invoked from the context unless and until it is
     * activated again.  The callback on_deactivate shall be called as
     * a result of calling this operation. This operation shall not
     * return until the callback has returned, and shall result in an
     * error if the callback does.
     *
     * @param comp The target RT-Component for deactivate
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t deactivateComponent(RTC::LightweightRTObject_ptr comp,
                                          RTObjectStateMachine*& rtobj);
    RTC::ReturnCode_t waitDeactivateComplete(RTObjectStateMachine*& rtobj,
                                             coil::TimeValue timeout = 1.0,
                                             long int cycle = 1000);

    /*!
     * @if jp
     * @brief RTݡͥȤꥻåȤ
     *
     * Error ֤RTݡͥȤߤ롣ƤФ줿
     * ̡on_reset ƤӽФ롣ꤷRTݡͥȤüԥ
     * Ȥ˴ޤޤʤϡBAD_PARAMETER ֤롣ꤷRT
     * ݡͥȤξ֤ Error ʳξϡPRECONDITION_NOT_MET 
     * 롣
     *
     * @param comp ꥻåоRTݡͥ
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Reset the RT-component
     *
     * Attempt to recover the RTC when it is in Error.  The
     * ComponentAction::on_reset callback shall be invoked. This
     * operation shall not return until the callback has returned, and
     * shall result in an error if the callback does. If possible, the
     * RTC developer should implement that callback such that the RTC
     * may be returned to a valid state.
     *
     * @param comp The target RT-Component for reset
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t resetComponent(RTC::LightweightRTObject_ptr com,
                                     RTObjectStateMachine*& rtobj);
    RTC::ReturnCode_t waitResetComplete(RTObjectStateMachine*& rtobj,
                                        coil::TimeValue timeout = 1.0,
                                        long int cycle = 1000);

    /*!
     * @if jp
     * @brief RTݡͥȤξ֤
     *
     * ꤷRTݡͥȤξ(LifeCycleState)롣ꤷ
     * RTݡͥȤüԥꥹȤ˴ޤޤʤϡ
     * UNKNOWN_STATE ֤롣
     *
     * @param comp ּоRTݡͥ
     *
     * @return ߤξ(LifeCycleState)
     *
     * @else
     *
     * @brief Get RT-component's state
     *
     * This operation shall report the LifeCycleState of the given
     * participant RTC.  UNKNOWN_STATE will be returned, if the given
     * RT-Component is not inclued in the participant list.
     *
     * @param comp The target RT-Component to get the state
     *
     * @return The current state of the target RT-Component(LifeCycleState)
     *
     * @endif
     */
    RTC::LifeCycleState getComponentState(RTC::LightweightRTObject_ptr comp);
    const char* getStateString(RTC::LifeCycleState state)
    {
      const char* st[] = {
        "CREATED_STATE",
        "INACTIVE_STATE",
        "ACTIVE_STATE",
        "ERROR_STATE"
      };
      return state >= RTC::CREATED_STATE && state <= RTC::ERROR_STATE ?
        st[state] : "";
    }
    /*!
     * @if jp
     * @brief RTݡͥȤɲä
     *
     * ꤷRTݡͥȤ"üԥꥹ"ɲä롣ɲä줿
     * RTݡͥȤ attach_context ƤФ졢Inactive ֤
     * 롣ꤵ줿RTݡͥȤ null ξϡBAD_PARAMETER 
     * ֤롣ꤵ줿RTݡͥȤ DataFlowComponent ʳ
     * ϡBAD_PARAMETER ֤롣
     *
     * δؿƤФ줿ʳǤϡ¾ RTC  StateMachine Υ
     * ХåؿƤӽФβǽꡢޤ "üԥꥹ:
     * m_addedComps"  RTC ɲäƤХåƤӽФö
     * äʳƱŪ updateComponentList() ؿƤФ졢
     * Worker  RTC ä֤Ȥʤ롣
     *
     * @param comp ɲоRTݡͥ
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Add an RT-component
     *
     * The operation causes the given RTC to begin participating in
     * the execution context.  The newly added RTC will receive a call
     * to LightweightRTComponent::attach_context and then enter the
     * Inactive state.  BAD_PARAMETER will be invoked, if the given
     * RT-Component is null or if the given RT-Component is other than
     * DataFlowComponent.
     *
     * When right after this operation is called, since other RTCs
     * might being called their callbacks of the StateMachine, at
     * first the RTC is added into the "Participants: m_addedComps."
     * After RTCs callbacks are called, updateComponentList() function
     * is invoked and RTCs are officially a member of the worker.
     *
     * @param comp The target RT-Component for add
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t addComponent(RTC::LightweightRTObject_ptr comp);

    /*!
     * @if jp
     * @brief ݡͥȤХɤ롣
     *
     * ݡͥȤ owned context Хɤ롣Ϥ
     * RTObject_impl ϡ EC  owner ǤꡢŪˤ RTObject
     *  bindContext() 򥳡ᡢec_id롣ec_id 
     * ECOTHER_OFFSET ⾮θ塢RTObject ΥץǤ
     * RTObjectStateMachine ݡͥȤΥꥹȤ˲ä롣
     *
     * @param rtc RTݡͥ
     * @return ReturnCode_t Υ꥿󥳡
     * @else
     * @brief Bind the component.
     *
     * This operation bind a RT-Component into owned context. Given
     * argument RTObject_impl is the owner of the EC, and internally
     * RTObject's bindContext() member function is called and obtain
     * ec_id. The ec_id must be smaller than ECOTHER_OFFSET. After
     * that proxy of RTObject RTObjectStateMachine is created, and it
     * is added to the component list.
     *
     * @param rtc RT-Component's instances
     * @return The return code of ReturnCode_t type
     * @endif
     */
    RTC::ReturnCode_t bindComponent(RTC::RTObject_impl* rtc);

    /*!
     * @if jp
     * @brief RTݡͥȤ򻲲üԥꥹȤ
     *
     * ꤷRTݡͥȤ򻲲üԥꥹȤ롣줿
     * RTݡͥȤ detach_context ƤФ롣ꤵ줿RTݡ
     * ͥȤüԥꥹȤϿƤʤϡBAD_PARAMETER 
     * 롣
     *
     * @param comp оRTݡͥ
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Remove the RT-Component from participant list
     *
     * This operation causes a participant RTC to stop participating in the
     * execution context.
     * The removed RTC will receive a call to
     * LightweightRTComponent::detach_context.
     * BAD_PARAMETER will be returned, if the given RT-Component is not 
     * participating in the participant list.
     *
     * @param comp The target RT-Component for delete
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t removeComponent(RTC::LightweightRTObject_ptr comp);
    RTObjectStateMachine* findComponent(RTC::LightweightRTObject_ptr comp);

    bool isAllCurrentState(RTC::LifeCycleState state);
    bool isAllNextState(RTC::LifeCycleState state);
    bool isOneOfCurrentState(RTC::LifeCycleState state);
    bool isOneOfNextState(RTC::LifeCycleState state);

    void invokeWorker();
    void invokeWorkerPreDo();
    void invokeWorkerDo();
    void invokeWorkerPostDo();

    /*!
     * @if jp
     * @brief ݡͥȥꥹȤι
     * @else
     * @brief Updating component list
     * @endif
     */
    void updateComponentList();

    //------------------------------------------------------------
    // member variables
  protected:
    /*!
     * @if jp
     * @brief ȥ꡼
     * @else
     * @brief Logger stream
     * @endif
     */
    RTC::Logger rtclog;

    RTC::ExecutionContextService_var m_ref;

    /*!
     * @if jp
     * @brief ExecutionContext μ¹Ծ
     * true: running, false: stopped
     * @else
     * @brief The running state of ExecutionContext
     * true: running, false: stopped
     * @endif
     */
    bool m_running;

    /*!
     * @if jp
     * @brief ݡͥȤλüԥꥹ
     * @else
     * @brief List of the participating component
     * @endif
     */
    std::vector<RTC_impl::RTObjectStateMachine*> m_comps;
    mutable coil::Mutex m_mutex;
    std::vector<RTC_impl::RTObjectStateMachine*> m_addedComps;
    mutable coil::Mutex m_addedMutex;
    std::vector<RTC_impl::RTObjectStateMachine*> m_removedComps;
    mutable coil::Mutex m_removedMutex;
    typedef std::vector<RTC_impl::RTObjectStateMachine*>::iterator CompItr;

  }; // class PeriodicExecutionContext
}; // namespace RTC

#ifdef WIN32
#pragma warning( default : 4290 )
#endif

#endif // RTC_PERIODICEXECUTIONCONTEXT_H
