// -*- C++ -*-
/*!
 * @file ExtTrigExecutionContext.h
 * @brief ExtTrigExecutionContext class
 * @date $Date: 2008-01-14 07:49:16 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2007,2012
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: ExtTrigExecutionContext.h 3097 2017-12-09 23:48:36Z n-ando $
 *
 */

#ifndef RTC_EXTTRIGEXECUTIONCONTEXT_H
#define RTC_EXTTRIGEXECUTIONCONTEXT_H

#include <rtm/RTC.h>

#include <coil/Mutex.h>
#include <coil/Condition.h>
#include <coil/Task.h>

#include <rtm/ExecutionContextBase.h>

#ifdef WIN32
#pragma warning( disable : 4290 )
#endif

namespace RTC
{
  /*!
   * @if jp
   * @class ExtTrigExecutionContext
   * @brief ƥå׼¹Ԥǽ ExecutionContext 饹
   *
   * 1μ¹ԤǽPeriodic Sampled Data Processing(¹)
   * ExecutionContext饹
   * Υ᥽åɸƤӤˤäƻ֤1ŤĿʤࡣ
   *
   * @since 0.4.0
   *
   * @else
   * @brief ExecutionContext class that enables one step execution
   *
   * ExecutionContext class that can execute every one cycle for Periodic
   * Sampled Data Processing.
   * Time(Tick) advances one cycle by invoking method externally.
   *
   * @since 0.4.0
   *
   * @endif
   */
  class ExtTrigExecutionContext
    : public virtual POA_OpenRTM::ExtTrigExecutionContextService,
      public virtual PortableServer::RefCountServantBase,
      public RTC::ExecutionContextBase,
      public coil::Task
  {
    typedef coil::Mutex Mutex;
    typedef coil::Condition<Mutex> Condition;
    typedef coil::Guard<coil::Mutex> Guard;
  public:
    /*!
     * @if jp
     * @brief 󥹥ȥ饯
     *
     * 󥹥ȥ饯
     *
     * @else
     * @brief Constructor
     *
     * Constructor
     *
     * @endif
     */
    ExtTrigExecutionContext();
    
    /*!
     * @if jp
     * @brief ǥȥ饯
     *
     * ǥȥ饯
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~ExtTrigExecutionContext(void);
    
    /*!
     * @if jp
     * @brief ExecutionContextѥƥӥƥåɤ
     *
     * Executioncontext Ѥƥӥƥåɤư롣
     *  coil::Task ӥ饹᥽åɤΥС饤ɡ
     *
     * @param args ̾0
     *
     * @return ¹Է
     *
     * @else
     *
     * @brief Generate internal activity thread for ExecutionContext
     *
     * Generate internal activity thread and run.  This is coil::Task
     * class method's override.
     *
     * @param args Usually give 0
     *
     * @return The generation result
     *
     * @endif
     */
    virtual int open(void *args);

    /*!
     * @if jp
     * @brief  Component νƤӽФ
     *
     * ExecutionContext  attach Ƥ Component νƤӽФ
     *  Component νƤӽФ塢θƽФȯޤǵٻߤ롣
     *
     * @return 
     *
     * @else
     * @brief Invoke each component's operation
     *
     * Invoke each component's operation which is attached this
     * ExecutionContext.  Stop until the next operation is invoked
     * after all component operations are invoked.
     *
     * @return Operation result
     *
     * @endif
     */
    virtual int svc(void);

    /*!
     * @if jp
     * @brief ExecutionContext ѤΥåɼ¹Դؿ
     *
     * ExecutionContext ѤΥåɽλ˸ƤФ롣ݡͥȥ
     * ֥Ȥ󥢥ƥֲޥ͡ؤΤԤ
     * coil::Task ӥ饹᥽åɤΥС饤ɡ
     *
     * @param flags λե饰
     *
     * @return λ
     *
     * @else
     *
     * @brief Thread execution function for ExecutionContext
     *
     * This function is invoked when activity thread for
     * ExecutionContext exits.  Deactivate the component object and
     * notify it to manager.  This is coil::Task class method's
     * override.
     *
     * @param flags Flag of the close
     *
     * @return The close result
     *
     * @endif
     */
    virtual int close(unsigned long flags);

    //============================================================
    // ExtTrigExecutionContextService
    //============================================================
    /*!
     * @if jp
     * @brief 1ƥå׿ʤ
     *
     * ExecutionContextν1ʬʤ롣
     *
     * @else
     * @brief Move forward one step of ExecutionContext
     *
     * Move forward one step of the ExecutionContext processing.
     *
     * @endif
     */
    virtual void tick()
      throw (CORBA::SystemException);
    
    //============================================================
    // ExecutionContextService
    //============================================================
    /*!
     * @if jp
     * @brief ExecutionContext ¹Ծֳǧؿ
     *
     *  ExecutionContext  Runnning ֤ξ true ֤
     * Executioncontext  Running δ֡ Executioncontext ˻ä
     * ƤƤΥƥRTݡͥȤExecutionContext μ
     * Լ˱Ƽ¹Ԥ롣
     *
     * @return ֳǧؿ(ư:true:false)
     *
     * @else
     *
     * @brief Check for ExecutionContext running state
     *
     * This operation shall return true if the context is in the
     * Running state.  While the context is Running, all Active RTCs
     * participating in the context shall be executed according to the
     * contexts execution kind.
     *
     * @return Check state function (Running:trueStopping:false)
     *
     * @endif
     */
    virtual CORBA::Boolean is_running(void)
      throw (CORBA::SystemException);

    /*!
     * @if jp
     * @brief ExecutionContext μ¹Ԥ򳫻
     *
     * ExecutionContext μ¹Ծ֤ Runnning Ȥ뤿ΥꥯȤ
     * ȯԤ롣ExecutionContext ξ֤ܤ
     * ComponentAction::on_startup ƤӽФ롣äƤRTݡ
     * ͥȤޤ ExecutionContext 򳫻Ϥ뤳ȤϤǤ
     * ʤExecutionContext ʣ󳫻/ߤ򷫤֤ȤǤ롣
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Start the ExecutionContext
     *
     * Request that the context enter the Running state.  Once the
     * state transition occurs, the ComponentAction::on_startup
     * operation will be invoked.  An execution context may not be
     * started until the RT-Components that participate in it have
     * been initialized.  An execution context may be started and
     * stopped multiple times.
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual RTC::ReturnCode_t start(void)
      throw (CORBA::SystemException);

    /*!
     * @if jp
     * @brief ExecutionContext μ¹Ԥ
     *
     * ExecutionContext ξ֤ Stopped Ȥ뤿ΥꥯȤȯԤ
     * 롣ܤȯϡComponentAction::on_shutdown Ƥӽ
     * 롣äƤRTݡͥȤλ
     * ExecutionContext ߤɬפ롣ExecutionContext ʣ
     * /ߤ򷫤֤ȤǤ롣
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Stop the ExecutionContext
     *
     * Request that the context enter the Stopped state.  Once the
     * transition occurs, the ComponentAction::on_shutdown operation
     * will be invoked.  An execution context must be stopped before
     * the RT components that participate in it are finalized.  An
     * execution context may be started and stopped multiple times.
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual RTC::ReturnCode_t stop(void)
      throw (CORBA::SystemException);

    /*!
     * @if jp
     * @brief ExecutionContext μ¹Լ(Hz)
     *
     * Active ֤ˤRTݡͥȤ¹Ԥ(ñ:Hz)
     * 롣
     *
     * @return (ñ:Hz)
     *
     * @else
     *
     * @brief Get execution rate(Hz) of ExecutionContext
     *
     * This operation shall return the rate (in hertz) at which its
     * Active participating RTCs are being invoked.
     *
     * @return Execution cycle(Unit:Hz)
     *
     * @endif
     */
    virtual CORBA::Double get_rate(void)
      throw (CORBA::SystemException);

    /*!
     * @if jp
     * @brief ExecutionContext μ¹Լ(Hz)ꤹ
     *
     * Active ֤ˤRTݡͥȤ¹Ԥ(ñ:Hz)ꤹ
     * 롣¹ԼѹϡDataFlowComponentAction 
     * on_rate_changed ˤäƳRTݡͥȤã롣
     *
     * @param rate (ñ:Hz)
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Set execution rate(Hz) of ExecutionContext
     *
     * This operation shall set the rate (in hertz) at which this
     * contexts Active participating RTCs are being called.  If the
     * execution kind of the context is PERIODIC, a rate change shall
     * result in the invocation of on_rate_changed on any RTCs
     * realizing DataFlowComponentAction that are registered with any
     * RTCs participating in the context.
     *
     * @param rate Execution cycle(Unit:Hz)
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual RTC::ReturnCode_t  set_rate(CORBA::Double rate)
      throw (CORBA::SystemException);

    /*!
     * @if jp
     * @brief RTݡͥȤ򥢥ƥֲ
     *
     * Inactive ֤ˤRTݡͥȤActive ܤƥ
     * ֲ롣ƤФ줿̡on_activate ƤӽФ롣
     * ꤷRTݡͥȤüԥꥹȤ˴ޤޤʤϡ
     * BAD_PARAMETER ֤롣ꤷRTݡͥȤξ֤
     * Inactive ʳξϡPRECONDITION_NOT_MET ֤롣
     *
     * @param comp ƥֲоRTݡͥ
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Activate an RT-component
     *
     * The given participant RTC is Inactive and is therefore not
     * being invoked according to the execution contexts execution
     * kind. This operation shall cause the RTC to transition to the
     * Active state such that it may subsequently be invoked in this
     * execution context.  The callback on_activate shall be called as
     * a result of calling this operation. This operation shall not
     * return until the callback has returned, and shall result in an
     * error if the callback does.
     *
     * @param comp The target RT-Component for activation
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual RTC::ReturnCode_t
    activate_component(RTC::LightweightRTObject_ptr comp)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     * @brief RTݡͥȤ󥢥ƥֲ
     *
     * Inactive ֤ˤRTݡͥȤ󥢥ƥֲInactive
     * ܤ롣ƤФ줿̡on_deactivate ƤӽФ
     * 롣ꤷRTݡͥȤüԥꥹȤ˴ޤޤʤϡ
     * BAD_PARAMETER ֤롣ꤷRTݡͥȤξ֤
     * Active ʳξϡPRECONDITION_NOT_MET ֤롣
     *
     * @param comp 󥢥ƥֲоRTݡͥ
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Deactivate an RT-component
     *
     * The given RTC is Active in the execution context. Cause it to
     * transition to the Inactive state such that it will not be
     * subsequently invoked from the context unless and until it is
     * activated again.  The callback on_deactivate shall be called as
     * a result of calling this operation. This operation shall not
     * return until the callback has returned, and shall result in an
     * error if the callback does.
     *
     * @param comp The target RT-Component for deactivate
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual RTC::ReturnCode_t
    deactivate_component(RTC::LightweightRTObject_ptr comp)
      throw (CORBA::SystemException);

    /*!
     * @if jp
     * @brief RTݡͥȤꥻåȤ
     *
     * Error ֤RTݡͥȤߤ롣ƤФ줿
     * ̡on_reset ƤӽФ롣ꤷRTݡͥȤüԥ
     * Ȥ˴ޤޤʤϡBAD_PARAMETER ֤롣ꤷRT
     * ݡͥȤξ֤ Error ʳξϡPRECONDITION_NOT_MET 
     * 롣
     *
     * @param comp ꥻåоRTݡͥ
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Reset the RT-component
     *
     * Attempt to recover the RTC when it is in Error.  The
     * ComponentAction::on_reset callback shall be invoked. This
     * operation shall not return until the callback has returned, and
     * shall result in an error if the callback does. If possible, the
     * RTC developer should implement that callback such that the RTC
     * may be returned to a valid state.
     *
     * @param comp The target RT-Component for reset
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual RTC::ReturnCode_t
    reset_component(RTC::LightweightRTObject_ptr comp)
      throw (CORBA::SystemException);

    /*!
     * @if jp
     * @brief RTݡͥȤξ֤
     *
     * ꤷRTݡͥȤξ(LifeCycleState)롣ꤷ
     * RTݡͥȤüԥꥹȤ˴ޤޤʤϡ
     * UNKNOWN_STATE ֤롣
     *
     * @param comp ּоRTݡͥ
     *
     * @return ߤξ(LifeCycleState)
     *
     * @else
     *
     * @brief Get RT-component's state
     *
     * This operation shall report the LifeCycleState of the given
     * participant RTC.  UNKNOWN_STATE will be returned, if the given
     * RT-Component is not inclued in the participant list.
     *
     * @param comp The target RT-Component to get the state
     *
     * @return The current state of the target RT-Component(LifeCycleState)
     *
     * @endif
     */
    virtual RTC::LifeCycleState
    get_component_state(RTC::LightweightRTObject_ptr comp)
      throw (CORBA::SystemException);

    /*!
     * @if jp
     * @brief ExecutionKind 
     *
     *  ExecutionContext  ExecutionKind 
     *
     * @return ExecutionKind
     *
     * @else
     *
     * @brief Get the ExecutionKind
     *
     * This operation shall report the execution kind of the execution
     * context.
     *
     * @return ExecutionKind
     *
     * @endif
     */
    virtual RTC::ExecutionKind get_kind(void)
      throw (CORBA::SystemException);

    /*!
     * @if jp
     * @brief RTݡͥȤɲä
     *
     * ꤷRTݡͥȤ򻲲üԥꥹȤɲä롣ɲä줿RT
     * ݡͥȤ attach_context ƤФ졢Inactive ֤ܤ롣
     * ꤵ줿RTݡͥȤnullξϡBAD_PARAMETER ֤
     * 롣ꤵ줿RTݡͥȤ DataFlowComponent ʳξϡ
     * BAD_PARAMETER ֤롣
     *
     * @param comp ɲоRTݡͥ
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Add an RT-component
     *
     * The operation causes the given RTC to begin participating in
     * the execution context.  The newly added RTC will receive a call
     * to LightweightRTComponent::attach_context and then enter the
     * Inactive state.  BAD_PARAMETER will be invoked, if the given
     * RT-Component is null or if the given RT-Component is other than
     * DataFlowComponent.
     *
     * @param comp The target RT-Component for add
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual RTC::ReturnCode_t add_component(RTC::LightweightRTObject_ptr comp)
      throw (CORBA::SystemException);

    /*!
     * @if jp
     * @brief RTݡͥȤ򻲲üԥꥹȤ
     *
     * ꤷRTݡͥȤ򻲲üԥꥹȤ롣줿
     * RTݡͥȤ detach_context ƤФ롣ꤵ줿RTݡ
     * ͥȤüԥꥹȤϿƤʤϡBAD_PARAMETER 
     * 롣
     *
     * @param comp оRTݡͥ
     *
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Remove the RT-Component from participant list
     *
     * This operation causes a participant RTC to stop participating in the
     * execution context.
     * The removed RTC will receive a call to
     * LightweightRTComponent::detach_context.
     * BAD_PARAMETER will be returned, if the given RT-Component is not
     * participating in the participant list.
     *
     * @param comp The target RT-Component for delete
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual RTC::ReturnCode_t
    remove_component(RTC::LightweightRTObject_ptr comp)
      throw (CORBA::SystemException);

    /*!
     * @if jp
     * @brief ExecutionContextProfile 
     *
     *  ExecutionContext Υץե롣
     *
     * @return ExecutionContextProfile
     *
     * @else
     *
     * @brief Get the ExecutionContextProfile
     *
     * This operation provides a profile descriptor for the execution
     * context.
     *
     * @return ExecutionContextProfile
     *
     * @endif
     */
    virtual RTC::ExecutionContextProfile* get_profile(void)
      throw (CORBA::SystemException);

  protected:
    /*!
     * @brief onStarted() template function
     */
    virtual RTC::ReturnCode_t onStarted();
    // template virtual functions adding/removing component	
    /*!	
     * @brief onAddedComponent() template function	
     */	
     virtual RTC::ReturnCode_t	
     onAddedComponent(RTC::LightweightRTObject_ptr rtobj);	
    /*!	
     * @brief onRemovedComponent() template function	
     */	
    virtual RTC::ReturnCode_t	
    onRemovedComponent(RTC::LightweightRTObject_ptr rtobj);
    /*!
     * @brief onWaitingActivated() template function
     */
    virtual RTC::ReturnCode_t
    onWaitingActivated(RTC_impl::RTObjectStateMachine* comp, long int count);
    /*!
     * @brief onWaitingDeactivated() template function
     */
    virtual RTC::ReturnCode_t
    onWaitingDeactivated(RTC_impl::RTObjectStateMachine* comp, long int count);
    /*!
     * @brief onWaitingReset() template function
     */
    virtual RTC::ReturnCode_t
    onWaitingReset(RTC_impl::RTObjectStateMachine* comp, long int count);

  private:
    bool threadRunning()
    {
      Guard guard(m_svcmutex);
      return m_svc;
    }
    /*!
     * @if jp
     * @brief ȥ꡼
     * @else
     * @brief Logger stream
     * @endif
     */
    RTC::Logger rtclog;

    /*!
     * @if jp
     * @brief ExecutionContext Υåɼ¹ԥե饰
     * @else
     * @brief The thread running flag of ExecutionContext
     * @endif
     */
    bool m_svc;
    Mutex m_svcmutex;

    /*!
     * @if jp
     * @brief worker Ѿѿ饹
     * @else
     * @brief Condition variable class for worker
     * @endif
     */
    struct Worker
    {
      Worker() : cond_(mutex_), ticked_(false) {};
      Mutex mutex_;
      Condition cond_;
      bool ticked_;
    };
    // A condition variable for external triggered worker
    Worker m_worker;
  };  // class ExtTrigExecutionContext
};  // namespace RTC

#ifdef WIN32
#pragma warning( default : 4290 )
#endif


extern "C"
{
  /*!
   * @if jp
   * @brief  ExecutionContext Factory饹Ͽ
   *
   * ExecutionContextFactory饹
   * ExecutionContextObjectManagerϿ롣
   *
   * @else
   * @brief Register Factory class for this ExecutionContext
   *
   * Register the Factory class to create this ExecutionContext
   * to the ObjectManager for management ExecutionContext
   *
   * @endif
   */
  void ExtTrigExecutionContextInit(RTC::Manager* manager);
};

#endif // RTC_EXTTRIGEXECUTIONCONTEXT_H
